unit Adler32;

{ZLib - Adler32 checksum function}


interface

(*************************************************************************

 DESCRIPTION     :  ZLib - Adler32 checksum function

 REQUIREMENTS    :  D4-D7/D9-D10/D12, FPC

 EXTERNAL DATA   :  ---

 MEMORY USAGE    :  ---

 DISPLAY MODE    :  ---

 REFERENCES      :  RFC 1950 (http://tools.ietf.org/html/rfc1950)


 Version  Date      Author      Modification
 -------  --------  -------     ------------------------------------------
 0.10     29.09.07  W.Ehrhardt  Initial version based on standard v3.05
 0.11     12.11.08  we          uses BTypes, Ptr2Inc and/or Str255/Str127
 0.12     25.04.09  we          updated RFC URL(s)
 0.13     19.07.09  we          D12 fix: assign with typecast string(fname)
**************************************************************************)

(*-------------------------------------------------------------------------
 (C) Copyright 2002-2009 Wolfgang Ehrhardt

 This software is provided 'as-is', without any express or implied warranty.
 In no event will the authors be held liable for any damages arising from
 the use of this software.

 Permission is granted to anyone to use this software for any purpose,
 including commercial applications, and to alter it and redistribute it
 freely, subject to the following restrictions:

 1. The origin of this software must not be misrepresented; you must not
    claim that you wrote the original software. If you use this software in
    a product, an acknowledgment in the product documentation would be
    appreciated but is not required.

 2. Altered source versions must be plainly marked as such, and must not be
    misrepresented as being the original software.

 3. This notice may not be removed or altered from any source distribution.
----------------------------------------------------------------------------*)


{$i STD.INC}

uses
  BTypes;

procedure Adler32Init(var adler: longint);
  {-Adler32 initialization}
  {$ifdef DLL} stdcall; {$endif}

procedure Adler32Update(var adler: longint; Msg: pointer; Len: word);
  {-update Adler32 with Msg data}
  {$ifdef DLL} stdcall; {$endif}

procedure Adler32Final(var adler: longint);
  {-finalize Adler32 calculation, clear context}
  {$ifdef DLL} stdcall; {$endif}

function  Adler32SelfTest: boolean;
  {-Self test for Adler32}
  {$ifdef DLL} stdcall; {$endif}

procedure Adler32Full(var adler: longint; Msg: pointer; Len: word);
  {-Adler32 of Msg with Init/Update/Final}
  {$ifdef DLL} stdcall; {$endif}

procedure Adler32File(const fname: Str255; var adler: longint; var buf; bsize: word; var Err: word);
  {-Adler32 of file, buf: buffer with at least bsize bytes}
  {$ifdef DLL} stdcall; {$endif}

procedure Adler32UpdateXL(var adler: longint; Msg: pointer; Len: longint);
  {-update Adler32 with Msg data}
  {$ifdef DLL} stdcall; {$endif}

procedure Adler32FullXL(var adler: longint; Msg: pointer; Len: longint);
  {-Adler32 of Msg with Init/Update/Final}
  {$ifdef DLL} stdcall; {$endif}


implementation


{---------------------------------------------------------------------------}
procedure Adler32Init(var adler: longint);
  {-Adler32 initialization}
begin
  adler := 1;
end;




{---------------------------------------------------------------------------}
procedure Adler32UpdateXL(var adler: longint; Msg: pointer; Len: longint);
  {-update Adler32 with Msg data}
const
  BASE = 65521; {max. prime < 65536 }
  NMAX =  3854; {max. n with 255n(n+1)/2 + (n+1)(BASE-1) < 2^31}

var
  s1,s2: longint;
  i,n: integer;
begin
  s1 := adler and $FFFF;
  s2 := adler shr 16;
  while Len > 0 do begin
    if Len<NMAX then n := Len else n := NMAX;
    for i:=1 to n do begin
      inc(s1, pByte(Msg)^);
      inc(Ptr2Inc(Msg));
      inc(s2, s1);
    end;
    s1 := s1 mod BASE;
    s2 := s2 mod BASE;
    dec(len, n);
  end;
  adler := (s2 shl 16) or s1;
end;


{---------------------------------------------------------------------------}
procedure Adler32Update(var adler: longint; Msg: pointer; Len: word);
  {-update Adler32 with Msg data}
begin
  Adler32UpdateXL(adler, Msg, Len);
end;



{---------------------------------------------------------------------------}
procedure Adler32Final(var adler: longint);
  {-finalize Adler32 calculation, clear context}
begin
  {Dummy}
end;


{---------------------------------------------------------------------------}
function Adler32SelfTest: boolean;
  {-Self test for Adler32}
const
  vec: array[1..3] of byte = ($11,$22,$33);
var
  i: integer;
  A1, AF: longint;
begin
  Adler32Full(AF, @vec, sizeof(vec));
  Adler32Init(A1);
  for i:=1 to sizeof(vec) do Adler32Update(A1, @vec[i], 1);
  Adler32Final(A1);
  Adler32SelfTest := (A1=$00AD0067) and (AF=$00AD0067);
end;


{---------------------------------------------------------------------------}
procedure Adler32FullXL(var adler: longint; Msg: pointer; Len: longint);
  {-Adler32 of Msg with Init/Update/Final}
begin
  Adler32Init(adler);
  Adler32UpdateXL(adler, Msg, Len);
  Adler32Final(adler);
end;


{---------------------------------------------------------------------------}
procedure Adler32Full(var adler: longint; Msg: pointer; Len: word);
  {-Adler32 of Msg with Init/Update/Final}
begin
  Adler32FullXL(adler, Msg, Len);
end;


{$i-} {Force I-}
{---------------------------------------------------------------------------}
procedure Adler32File(const fname: Str255; var adler: longint; var buf; bsize: word; var Err: word);
  {-Adler32 of file, buf: buffer with at least bsize bytes}
var
  fms: byte;
  L: longint;
  f: file;
begin
  fms := FileMode;
  FileMode := 0;
  system.assign(f,{$ifdef D12Plus} string {$endif} (fname));
  system.reset(f,1);
  Err := IOResult;
  FileMode := fms;
  if Err<>0 then exit;
  Adler32Init(adler);
  L := bsize;
  while (Err=0) and (L=bsize) do begin
    system.blockread(f,buf,bsize,L);
    Err := IOResult;
    Adler32UpdateXL(adler, @buf, L);
  end;
  system.close(f);
  if IOResult=0 then;
  Adler32Final(adler);
end;


end.
